@@ -16,8 +16,9 @@
 
 """The share snapshots api."""
 
+from http import client as http_client
+
 from oslo_log import log
-from six.moves import http_client
 import webob
 from webob import exc
 
@@ -27,6 +28,7 @@ from manila.api.openstack import wsgi
 from manila.api.v1 import share_snapshots
 from manila.api.views import share_snapshots as snapshot_views
 from manila.common import constants
+from manila.db import api as db_api
 from manila import exception
 from manila.i18n import _
 from manila import share
@@ -75,7 +77,7 @@ class ShareSnapshotsController(share_snapshots.ShareSnapshotMixin,
 
             self.share_api.unmanage_snapshot(context, snapshot, share['host'])
         except (exception.ShareSnapshotNotFound, exception.ShareNotFound) as e:
-            raise exc.HTTPNotFound(explanation=e)
+            raise exc.HTTPNotFound(explanation=e.msg)
 
         return webob.Response(status_int=http_client.ACCEPTED)
 
@@ -114,23 +116,34 @@ class ShareSnapshotsController(share_snapshots.ShareSnapshotMixin,
         description = snapshot_data.get(
             'display_description', snapshot_data.get('description'))
 
+        share_id = snapshot_data['share_id']
         snapshot = {
-            'share_id': snapshot_data['share_id'],
+            'share_id': share_id,
             'provider_location': snapshot_data['provider_location'],
             'display_name': name,
             'display_description': description,
         }
 
+        try:
+            share_ref = self.share_api.get(context, share_id)
+        except exception.NotFound:
+            raise exception.ShareNotFound(share_id=share_id)
+        if share_ref.get('is_soft_deleted'):
+            msg = _("Can not manage snapshot for share '%s' "
+                    "since it has been soft deleted.") % share_id
+            raise exc.HTTPForbidden(explanation=msg)
+
         driver_options = snapshot_data.get('driver_options', {})
 
         try:
             snapshot_ref = self.share_api.manage_snapshot(context, snapshot,
-                                                          driver_options)
+                                                          driver_options,
+                                                          share=share_ref)
         except (exception.ShareNotFound, exception.ShareSnapshotNotFound) as e:
-            raise exc.HTTPNotFound(explanation=e)
+            raise exc.HTTPNotFound(explanation=e.msg)
         except (exception.InvalidShare,
                 exception.ManageInvalidShareSnapshot) as e:
-            raise exc.HTTPConflict(explanation=e)
+            raise exc.HTTPConflict(explanation=e.msg)
 
         return self._view_builder.detail(req, snapshot_ref)
 
@@ -161,6 +174,16 @@ class ShareSnapshotsController(share_snapshots.ShareSnapshotMixin,
             if not data.get(parameter):
                 msg = _("Required parameter %s is empty.") % parameter
                 raise exc_response(explanation=msg)
+            if not isinstance(data[parameter], str):
+                msg = _("Parameter %s must be a string.") % parameter
+                raise exc_response(explanation=msg)
+
+    def _check_if_share_share_network_is_active(self, context, snapshot):
+        share_network_id = snapshot['share'].get('share_network_id')
+        if share_network_id:
+            share_network = db_api.share_network_get(
+                context, share_network_id)
+            common.check_share_network_is_active(share_network)
 
     def _allow(self, req, id, body, enable_ipv6=False):
         context = req.environ['manila.context']
@@ -184,6 +207,8 @@ class ShareSnapshotsController(share_snapshots.ShareSnapshotMixin,
 
         snapshot = self.share_api.get_snapshot(context, id)
 
+        self._check_if_share_share_network_is_active(context, snapshot)
+
         self._check_mount_snapshot_support(context, snapshot)
 
         try:
@@ -212,6 +237,8 @@ class ShareSnapshotsController(share_snapshots.ShareSnapshotMixin,
 
         self._check_mount_snapshot_support(context, snapshot)
 
+        self._check_if_share_share_network_is_active(context, snapshot)
+
         access = self.share_api.snapshot_access_get(context, access_id)
 
         if access['share_snapshot_id'] != snapshot['id']:
