@@ -202,11 +202,12 @@ class ShareSnapshotAPITest(test.TestCase):
         req = fakes.HTTPRequest.blank(
             url, use_admin_context=use_admin_context, version=version)
 
-        snapshots = [
+        db_snapshots = [
             {'id': 'id1', 'display_name': 'n1', 'status': 'fake_status', },
             {'id': 'id2', 'display_name': 'n2', 'status': 'fake_status', },
             {'id': 'id3', 'display_name': 'n3', 'status': 'fake_status', },
         ]
+        snapshots = [db_snapshots[1]]
         self.mock_object(share_api.API, 'get_all_snapshots',
                          mock.Mock(return_value=snapshots))
 
@@ -225,14 +226,16 @@ class ShareSnapshotAPITest(test.TestCase):
             search_opts_expected.update({'fake_key': 'fake_value'})
         share_api.API.get_all_snapshots.assert_called_once_with(
             req.environ['manila.context'],
+            limit=int(search_opts['limit']),
+            offset=int(search_opts['offset']),
             sort_key=search_opts['sort_key'],
             sort_dir=search_opts['sort_dir'],
             search_opts=search_opts_expected,
         )
         self.assertEqual(1, len(result['snapshots']))
-        self.assertEqual(snapshots[1]['id'], result['snapshots'][0]['id'])
+        self.assertEqual(snapshots[0]['id'], result['snapshots'][0]['id'])
         self.assertEqual(
-            snapshots[1]['display_name'], result['snapshots'][0]['name'])
+            snapshots[0]['display_name'], result['snapshots'][0]['name'])
 
     @ddt.data({'version': '2.35', 'use_admin_context': True},
               {'version': '2.36', 'use_admin_context': True},
@@ -252,7 +255,7 @@ class ShareSnapshotAPITest(test.TestCase):
             url = url + '&' + k + '=' + v
         req = fakes.HTTPRequest.blank(url, use_admin_context=use_admin_context)
 
-        snapshots = [
+        db_snapshots = [
             {
                 'id': 'id1',
                 'display_name': 'n1',
@@ -273,6 +276,7 @@ class ShareSnapshotAPITest(test.TestCase):
                 'aggregate_status': 'fake_status',
             },
         ]
+        snapshots = [db_snapshots[1]]
 
         self.mock_object(share_api.API, 'get_all_snapshots',
                          mock.Mock(return_value=snapshots))
@@ -288,18 +292,20 @@ class ShareSnapshotAPITest(test.TestCase):
             search_opts_expected.update({'fake_key': 'fake_value'})
         share_api.API.get_all_snapshots.assert_called_once_with(
             req.environ['manila.context'],
+            limit=int(search_opts['limit']),
+            offset=int(search_opts['offset']),
             sort_key=search_opts['sort_key'],
             sort_dir=search_opts['sort_dir'],
             search_opts=search_opts_expected,
         )
         self.assertEqual(1, len(result['snapshots']))
-        self.assertEqual(snapshots[1]['id'], result['snapshots'][0]['id'])
+        self.assertEqual(snapshots[0]['id'], result['snapshots'][0]['id'])
         self.assertEqual(
-            snapshots[1]['display_name'], result['snapshots'][0]['name'])
+            snapshots[0]['display_name'], result['snapshots'][0]['name'])
         self.assertEqual(
-            snapshots[1]['aggregate_status'], result['snapshots'][0]['status'])
+            snapshots[0]['aggregate_status'], result['snapshots'][0]['status'])
         self.assertEqual(
-            snapshots[1]['share_id'], result['snapshots'][0]['share_id'])
+            snapshots[0]['share_id'], result['snapshots'][0]['share_id'])
 
     def test_snapshot_list_detail_with_search_opts_by_non_admin(self):
         self._snapshot_list_detail_with_search_opts(use_admin_context=False)
@@ -695,7 +701,8 @@ class ShareSnapshotAdminActionsAPITest(test.TestCase):
         {},
         {'snapshots': {}},
         {'snapshot': get_fake_manage_body(share_id='xxxxxxxx')},
-        {'snapshot': get_fake_manage_body(provider_location='xxxxxxxx')}
+        {'snapshot': get_fake_manage_body(provider_location='xxxxxxxx')},
+        {'snapshot': {'provider_location': {'x': 'y'}, 'share_id': 'xyzzy'}},
     )
     def test_snapshot_manage_invalid_body(self, body):
         self.mock_policy_check = self.mock_object(
@@ -725,9 +732,14 @@ class ShareSnapshotAdminActionsAPITest(test.TestCase):
         data['snapshot']['share_id'] = 'fake'
         data['snapshot']['provider_location'] = 'fake_volume_snapshot_id'
         data['snapshot']['driver_options'] = {}
+        return_share = fake_share.fake_share(is_soft_deleted=False,
+                                             id='fake')
         return_snapshot = fake_share.fake_snapshot(
             create_instance=True, id='fake_snap',
             provider_location='fake_volume_snapshot_id')
+        self.mock_object(
+            share_api.API, 'get', mock.Mock(
+                return_value=return_share))
         self.mock_object(
             share_api.API, 'manage_snapshot', mock.Mock(
                 return_value=return_snapshot))
@@ -746,7 +758,8 @@ class ShareSnapshotAdminActionsAPITest(test.TestCase):
 
         actual_snapshot = actual_result['snapshot']
         share_api.API.manage_snapshot.assert_called_once_with(
-            mock.ANY, share_snapshot, data['snapshot']['driver_options'])
+            mock.ANY, share_snapshot, data['snapshot']['driver_options'],
+            share=return_share)
         self.assertEqual(return_snapshot['id'],
                          actual_result['snapshot']['id'])
         self.assertEqual('fake_volume_snapshot_id',
@@ -775,6 +788,11 @@ class ShareSnapshotAdminActionsAPITest(test.TestCase):
         body = get_fake_manage_body(
             share_id='fake', provider_location='fake_volume_snapshot_id',
             driver_options={})
+        return_share = fake_share.fake_share(is_soft_deleted=False,
+                                             id='fake')
+        self.mock_object(
+            share_api.API, 'get', mock.Mock(
+                return_value=return_share))
         self.mock_object(
             share_api.API, 'manage_snapshot', mock.Mock(
                 side_effect=exception_type))
@@ -792,6 +810,25 @@ class ShareSnapshotAdminActionsAPITest(test.TestCase):
             self.manage_request.environ['manila.context'],
             self.resource_name, 'manage_snapshot')
 
+    def test_manage_share_has_been_soft_deleted(self):
+        self.mock_policy_check = self.mock_object(
+            policy, 'check_policy', mock.Mock(return_value=True))
+        body = get_fake_manage_body(
+            share_id='fake', provider_location='fake_volume_snapshot_id',
+            driver_options={})
+        return_share = fake_share.fake_share(is_soft_deleted=True,
+                                             id='fake')
+        self.mock_object(
+            share_api.API, 'get', mock.Mock(
+                return_value=return_share))
+
+        self.assertRaises(webob.exc.HTTPForbidden,
+                          self.controller.manage,
+                          self.manage_request, body)
+        self.mock_policy_check.assert_called_once_with(
+            self.manage_request.environ['manila.context'],
+            self.resource_name, 'manage_snapshot')
+
     @ddt.data('1.0', '2.6', '2.11')
     def test_manage_version_not_found(self, version):
         body = get_fake_manage_body(
