@@ -26,6 +26,8 @@ from manila.share.drivers.netapp.dataontap.client import api as netapp_api
 from manila.share.drivers.netapp.dataontap.client import client_cmode
 from manila.share.drivers.netapp.dataontap.cluster_mode import data_motion
 from manila.share.drivers.netapp import options as na_opts
+from manila.share.drivers.netapp import utils as na_utils
+from manila.share import utils as share_utils
 from manila import test
 from manila.tests.share.drivers.netapp.dataontap import fakes as fake
 from manila.tests.share.drivers.netapp import fakes as na_fakes
@@ -64,6 +66,8 @@ class NetAppCDOTDataMotionTestCase(test.TestCase):
                           group=self.backend)
         CONF.set_override("netapp_server_port", 8866,
                           group=self.backend)
+        CONF.set_override("netapp_ssl_cert_path", "/etc/ssl/certs",
+                          group=self.backend)
 
     def test_get_client_for_backend(self):
         self.mock_object(data_motion, "get_backend_configuration",
@@ -74,7 +78,7 @@ class NetAppCDOTDataMotionTestCase(test.TestCase):
         self.mock_cmode_client.assert_called_once_with(
             hostname='fake.hostname', password='fake_password',
             username='fake_user', transport_type='https', port=8866,
-            trace=mock.ANY, vserver=None)
+            ssl_cert_path='/etc/ssl/certs', trace=mock.ANY, vserver=None)
 
     def test_get_client_for_backend_with_vserver(self):
         self.mock_object(data_motion, "get_backend_configuration",
@@ -88,7 +92,24 @@ class NetAppCDOTDataMotionTestCase(test.TestCase):
         self.mock_cmode_client.assert_called_once_with(
             hostname='fake.hostname', password='fake_password',
             username='fake_user', transport_type='https', port=8866,
-            trace=mock.ANY, vserver='fake_vserver')
+            ssl_cert_path='/etc/ssl/certs', trace=mock.ANY,
+            vserver='fake_vserver')
+
+    def test_get_client_for_host(self):
+        mock_extract_host = self.mock_object(
+            share_utils, 'extract_host',
+            mock.Mock(return_value=fake.BACKEND_NAME))
+        mock_get_client = self.mock_object(
+            data_motion, 'get_client_for_backend',
+            mock.Mock(return_value=self.mock_cmode_client))
+
+        returned_client = data_motion.get_client_for_host(
+            fake.HOST_NAME)
+
+        mock_extract_host.assert_called_once_with(
+            fake.HOST_NAME, level='backend_name')
+        mock_get_client.assert_called_once_with(fake.BACKEND_NAME)
+        self.assertEqual(returned_client, self.mock_cmode_client)
 
     def test_get_config_for_backend(self):
         self.mock_object(data_motion, "CONF")
@@ -196,22 +217,34 @@ class NetAppCDOTDataMotionSessionTestCase(test.TestCase):
             fake.BACKEND_NAME, vserver_name=fake.VSERVER1
         )
 
-    def test_create_snapmirror(self):
+    @ddt.data(True, False)
+    def test_create_snapmirror_mount(self, mount):
         mock_dest_client = mock.Mock()
         self.mock_object(data_motion, 'get_client_for_backend',
                          mock.Mock(return_value=mock_dest_client))
+        self.mock_object(self.dm_session, 'wait_for_mount_replica')
+        mock_backend_config = na_fakes.create_configuration()
+        mock_backend_config.netapp_mount_replica_timeout = 30
+        self.mock_object(data_motion, 'get_backend_configuration',
+                         mock.Mock(return_value=mock_backend_config))
 
         self.dm_session.create_snapmirror(self.fake_src_share,
-                                          self.fake_dest_share)
+                                          self.fake_dest_share,
+                                          'data_protection', mount=mount)
 
         mock_dest_client.create_snapmirror_vol.assert_called_once_with(
             mock.ANY, self.fake_src_vol_name, mock.ANY,
-            self.fake_dest_vol_name, schedule='hourly'
+            self.fake_dest_vol_name, 'data_protection', schedule='hourly'
         )
         mock_dest_client.initialize_snapmirror_vol.assert_called_once_with(
             mock.ANY, self.fake_src_vol_name, mock.ANY,
             self.fake_dest_vol_name
         )
+        if mount:
+            self.dm_session.wait_for_mount_replica.assert_called_once_with(
+                mock_dest_client, self.fake_dest_vol_name, timeout=30)
+        else:
+            self.dm_session.wait_for_mount_replica.assert_not_called()
 
     def test_create_snapmirror_svm(self):
         mock_dest_client = mock.Mock()
@@ -254,6 +287,12 @@ class NetAppCDOTDataMotionSessionTestCase(test.TestCase):
         self.mock_object(data_motion, 'get_client_for_backend',
                          mock.Mock(side_effect=[mock_dest_client,
                                                 mock_src_client]))
+        mock_backend_config = na_fakes.create_configuration()
+        mock_backend_config.netapp_snapmirror_release_timeout = 30
+        self.mock_object(data_motion, 'get_backend_configuration',
+                         mock.Mock(return_value=mock_backend_config))
+        mock_wait_for_snapmirror_release_vol = self.mock_object(
+            self.dm_session, 'wait_for_snapmirror_release_vol')
 
         self.dm_session.delete_snapmirror(self.fake_src_share,
                                           self.fake_dest_share)
@@ -266,9 +305,10 @@ class NetAppCDOTDataMotionSessionTestCase(test.TestCase):
             mock.ANY, self.fake_src_vol_name, mock.ANY,
             self.fake_dest_vol_name
         )
-        mock_src_client.release_snapmirror_vol.assert_called_once_with(
-            mock.ANY, self.fake_src_vol_name, mock.ANY,
-            self.fake_dest_vol_name
+        mock_wait_for_snapmirror_release_vol.assert_called_once_with(
+            self.source_vserver, self.dest_vserver, self.fake_src_vol_name,
+            self.fake_dest_vol_name, False, mock_src_client,
+            timeout=30
         )
 
     @ddt.data(True, False)
@@ -305,6 +345,12 @@ class NetAppCDOTDataMotionSessionTestCase(test.TestCase):
         self.mock_object(data_motion, 'get_client_for_backend',
                          mock.Mock(side_effect=[mock_dest_client,
                                                 mock_src_client]))
+        mock_backend_config = na_fakes.create_configuration()
+        mock_backend_config.netapp_snapmirror_release_timeout = 30
+        self.mock_object(data_motion, 'get_backend_configuration',
+                         mock.Mock(return_value=mock_backend_config))
+        mock_wait_for_snapmirror_release_vol = self.mock_object(
+            self.dm_session, 'wait_for_snapmirror_release_vol')
 
         self.dm_session.delete_snapmirror(self.fake_src_share,
                                           self.fake_dest_share)
@@ -317,9 +363,10 @@ class NetAppCDOTDataMotionSessionTestCase(test.TestCase):
             mock.ANY, self.fake_src_vol_name, mock.ANY,
             self.fake_dest_vol_name
         )
-        mock_src_client.release_snapmirror_vol.assert_called_once_with(
-            mock.ANY, self.fake_src_vol_name, mock.ANY,
-            self.fake_dest_vol_name
+        mock_wait_for_snapmirror_release_vol.assert_called_once_with(
+            self.source_vserver, self.dest_vserver, self.fake_src_vol_name,
+            self.fake_dest_vol_name, False, mock_src_client,
+            timeout=30
         )
 
     def test_delete_snapmirror_svm_does_not_exist(self):
@@ -356,6 +403,12 @@ class NetAppCDOTDataMotionSessionTestCase(test.TestCase):
         self.mock_object(data_motion, 'get_client_for_backend',
                          mock.Mock(side_effect=[mock_dest_client,
                                                 mock_src_client]))
+        mock_backend_config = na_fakes.create_configuration()
+        mock_backend_config.netapp_snapmirror_release_timeout = 30
+        self.mock_object(data_motion, 'get_backend_configuration',
+                         mock.Mock(return_value=mock_backend_config))
+        mock_wait_for_snapmirror_release_vol = self.mock_object(
+            self.dm_session, 'wait_for_snapmirror_release_vol')
 
         self.dm_session.delete_snapmirror(self.fake_src_share,
                                           self.fake_dest_share)
@@ -368,9 +421,10 @@ class NetAppCDOTDataMotionSessionTestCase(test.TestCase):
             mock.ANY, self.fake_src_vol_name, mock.ANY,
             self.fake_dest_vol_name
         )
-        mock_src_client.release_snapmirror_vol.assert_called_once_with(
-            mock.ANY, self.fake_src_vol_name, mock.ANY,
-            self.fake_dest_vol_name
+        mock_wait_for_snapmirror_release_vol.assert_called_once_with(
+            self.source_vserver, self.dest_vserver, self.fake_src_vol_name,
+            self.fake_dest_vol_name, False, mock_src_client,
+            timeout=30
         )
 
     def test_delete_snapmirror_svm_error_deleting(self):
@@ -398,38 +452,14 @@ class NetAppCDOTDataMotionSessionTestCase(test.TestCase):
             timeout=mock_backend_config.netapp_snapmirror_release_timeout
         )
 
-    def test_delete_snapmirror_error_releasing(self):
-        """Ensure delete succeeds when the snapmirror does not exist."""
-        mock_src_client = mock.Mock()
-        mock_dest_client = mock.Mock()
-        mock_src_client.release_snapmirror_vol.side_effect = (
-            netapp_api.NaApiError(code=netapp_api.EOBJECTNOTFOUND))
-        self.mock_object(data_motion, 'get_client_for_backend',
-                         mock.Mock(side_effect=[mock_dest_client,
-                                                mock_src_client]))
-
-        self.dm_session.delete_snapmirror(self.fake_src_share,
-                                          self.fake_dest_share)
-
-        mock_dest_client.abort_snapmirror_vol.assert_called_once_with(
-            mock.ANY, self.fake_src_vol_name, mock.ANY,
-            self.fake_dest_vol_name, clear_checkpoint=False
-        )
-        mock_dest_client.delete_snapmirror_vol.assert_called_once_with(
-            mock.ANY, self.fake_src_vol_name, mock.ANY,
-            self.fake_dest_vol_name
-        )
-        mock_src_client.release_snapmirror_vol.assert_called_once_with(
-            mock.ANY, self.fake_src_vol_name, mock.ANY,
-            self.fake_dest_vol_name
-        )
-
     def test_delete_snapmirror_without_release(self):
         mock_src_client = mock.Mock()
         mock_dest_client = mock.Mock()
         self.mock_object(data_motion, 'get_client_for_backend',
                          mock.Mock(side_effect=[mock_dest_client,
                                                 mock_src_client]))
+        mock_wait_for_snapmirror_release_vol = self.mock_object(
+            self.dm_session, 'wait_for_snapmirror_release_vol')
 
         self.dm_session.delete_snapmirror(self.fake_src_share,
                                           self.fake_dest_share,
@@ -443,14 +473,15 @@ class NetAppCDOTDataMotionSessionTestCase(test.TestCase):
             mock.ANY, self.fake_src_vol_name, mock.ANY,
             self.fake_dest_vol_name
         )
-        self.assertFalse(mock_src_client.release_snapmirror_vol.called)
+        self.assertFalse(mock_wait_for_snapmirror_release_vol.called)
 
     def test_delete_snapmirror_source_unreachable(self):
-        mock_src_client = mock.Mock()
         mock_dest_client = mock.Mock()
         self.mock_object(data_motion, 'get_client_for_backend',
                          mock.Mock(side_effect=[mock_dest_client,
                                                 Exception]))
+        mock_wait_for_snapmirror_release_vol = self.mock_object(
+            self.dm_session, 'wait_for_snapmirror_release_vol')
 
         self.dm_session.delete_snapmirror(self.fake_src_share,
                                           self.fake_dest_share)
@@ -463,8 +494,7 @@ class NetAppCDOTDataMotionSessionTestCase(test.TestCase):
             mock.ANY, self.fake_src_vol_name, mock.ANY,
             self.fake_dest_vol_name
         )
-
-        self.assertFalse(mock_src_client.release_snapmirror_vol.called)
+        self.assertFalse(mock_wait_for_snapmirror_release_vol.called)
 
     def test_break_snapmirror(self):
         self.mock_object(self.dm_session, 'quiesce_then_abort')
@@ -640,6 +670,8 @@ class NetAppCDOTDataMotionSessionTestCase(test.TestCase):
                                                 self.mock_src_client,
                                                 self.mock_dest_client,
                                                 mock_new_src_client]))
+        self.mock_object(na_utils, 'get_relationship_type',
+                         mock.Mock(return_value=na_utils.DATA_PROTECTION_TYPE))
 
         self.dm_session.change_snapmirror_source(
             self.fake_dest_share, self.fake_src_share, fake_new_src_share,
@@ -649,12 +681,14 @@ class NetAppCDOTDataMotionSessionTestCase(test.TestCase):
 
         self.assertEqual(4, self.dm_session.delete_snapmirror.call_count)
         self.dm_session.delete_snapmirror.assert_called_with(
-            mock.ANY, mock.ANY, release=False
+            mock.ANY, mock.ANY, release=False, relationship_info_only=False
         )
 
+        na_utils.get_relationship_type.assert_called_once_with(False)
         self.mock_dest_client.create_snapmirror_vol.assert_called_once_with(
             mock.ANY, fake_new_src_share_name, mock.ANY,
-            self.fake_dest_vol_name, schedule='hourly'
+            self.fake_dest_vol_name, na_utils.DATA_PROTECTION_TYPE,
+            schedule='hourly'
         )
 
         self.mock_dest_client.resync_snapmirror_vol.assert_called_once_with(
@@ -682,6 +716,8 @@ class NetAppCDOTDataMotionSessionTestCase(test.TestCase):
         peer_cluster_name = 'new_src_cluster_name'
         self.mock_object(self.mock_src_client, 'get_cluster_name',
                          mock.Mock(return_value=peer_cluster_name))
+        self.mock_object(na_utils, 'get_relationship_type',
+                         mock.Mock(return_value=na_utils.DATA_PROTECTION_TYPE))
 
         self.dm_session.change_snapmirror_source(
             self.fake_dest_share, self.fake_src_share, fake_new_src_share,
@@ -700,12 +736,14 @@ class NetAppCDOTDataMotionSessionTestCase(test.TestCase):
         self.mock_src_client.accept_vserver_peer.assert_called_once_with(
             fake_new_src_ss_name, self.dest_vserver
         )
+        na_utils.get_relationship_type.assert_called_once_with(False)
         self.dm_session.delete_snapmirror.assert_called_with(
-            mock.ANY, mock.ANY, release=False
+            mock.ANY, mock.ANY, release=False, relationship_info_only=False
         )
         self.mock_dest_client.create_snapmirror_vol.assert_called_once_with(
             mock.ANY, fake_new_src_share_name, mock.ANY,
-            self.fake_dest_vol_name, schedule='hourly'
+            self.fake_dest_vol_name, na_utils.DATA_PROTECTION_TYPE,
+            schedule='hourly'
         )
         self.mock_dest_client.resync_snapmirror_vol.assert_called_once_with(
             mock.ANY, fake_new_src_share_name, mock.ANY,
@@ -1023,13 +1061,8 @@ class NetAppCDOTDataMotionSessionTestCase(test.TestCase):
                       dest_vserver=fake.VSERVER2),
             mock.call(source_vserver=fake.VSERVER1,
                       dest_vserver=fake.VSERVER2)])
-        if release_snapmirror_ret.side_effect is None:
-            src_mock_client.release_snapmirror_svm.assert_called_once_with(
-                fake.VSERVER1, fake.VSERVER2)
-        else:
-            src_mock_client.release_snapmirror_svm.assert_called_once_with(
-                fake.VSERVER1, fake.VSERVER2
-            )
+        src_mock_client.release_snapmirror_svm.assert_called_once_with(
+            fake.VSERVER1, fake.VSERVER2)
 
     def test_wait_for_snapmirror_release_svm_timeout(self):
         src_mock_client = mock.Mock()
@@ -1048,3 +1081,155 @@ class NetAppCDOTDataMotionSessionTestCase(test.TestCase):
         src_mock_client.release_snapmirror_svm.assert_called_once_with(
             fake.VSERVER1, fake.VSERVER2
         )
+
+    def test_wait_for_mount_replica(self):
+
+        mock_client = mock.Mock()
+        self.mock_object(time, 'sleep')
+        mock_warning_log = self.mock_object(data_motion.LOG, 'warning')
+
+        self.dm_session.wait_for_mount_replica(
+            mock_client, fake.SHARE_NAME)
+
+        mock_client.mount_volume.ssert_called_once_with(fake.SHARE_NAME)
+        self.assertEqual(0, mock_warning_log.call_count)
+
+    def test_wait_for_mount_replica_timeout(self):
+
+        mock_client = mock.Mock()
+        self.mock_object(time, 'sleep')
+        mock_warning_log = self.mock_object(data_motion.LOG, 'warning')
+        undergoing_snapmirror = (
+            'The volume is undergoing a snapmirror initialize.')
+        na_api_error = netapp_api.NaApiError(code=netapp_api.EAPIERROR,
+                                             message=undergoing_snapmirror)
+        mock_client.mount_volume.side_effect = na_api_error
+
+        self.assertRaises(exception.NetAppException,
+                          self.dm_session.wait_for_mount_replica,
+                          mock_client, fake.SHARE_NAME, timeout=30)
+
+        self.assertEqual(3, mock_client.mount_volume.call_count)
+        self.assertEqual(3, mock_warning_log.call_count)
+
+    def test_wait_for_mount_replica_api_not_found(self):
+
+        mock_client = mock.Mock()
+        self.mock_object(time, 'sleep')
+        mock_warning_log = self.mock_object(data_motion.LOG, 'warning')
+        na_api_error = netapp_api.NaApiError(code=netapp_api.EOBJECTNOTFOUND)
+        mock_client.mount_volume.side_effect = na_api_error
+
+        self.assertRaises(exception.NetAppException,
+                          self.dm_session.wait_for_mount_replica,
+                          mock_client, fake.SHARE_NAME, timeout=30)
+
+        mock_client.mount_volume.assert_called_once_with(fake.SHARE_NAME)
+        mock_warning_log.assert_not_called()
+
+    @ddt.data(mock.Mock(),
+              mock.Mock(side_effect=netapp_api.NaApiError(
+                  code=netapp_api.EOBJECTNOTFOUND)))
+    def test_wait_for_snapmirror_release_vol(self, release_snapmirror_ret):
+        src_mock_client = mock.Mock()
+        get_snapmirrors_mock = self.mock_object(
+            src_mock_client, 'get_snapmirror_destinations',
+            mock.Mock(side_effect=[['fake_snapmirror'], []]))
+        self.mock_object(src_mock_client, 'release_snapmirror_vol',
+                         release_snapmirror_ret)
+
+        self.dm_session.wait_for_snapmirror_release_vol(fake.VSERVER1,
+                                                        fake.VSERVER2,
+                                                        fake.SHARE_NAME,
+                                                        fake.SHARE_NAME2,
+                                                        False,
+                                                        src_mock_client,
+                                                        timeout=20)
+        get_snapmirrors_mock.assert_has_calls([
+            mock.call(source_vserver=fake.VSERVER1,
+                      dest_vserver=fake.VSERVER2,
+                      source_volume=fake.SHARE_NAME,
+                      dest_volume=fake.SHARE_NAME2),
+            mock.call(source_vserver=fake.VSERVER1,
+                      dest_vserver=fake.VSERVER2,
+                      source_volume=fake.SHARE_NAME,
+                      dest_volume=fake.SHARE_NAME2)])
+        src_mock_client.release_snapmirror_vol.assert_called_once_with(
+            fake.VSERVER1, fake.SHARE_NAME, fake.VSERVER2, fake.SHARE_NAME2,
+            relationship_info_only=False)
+
+    def test_wait_for_snapmirror_release_vol_timeout(self):
+        src_mock_client = mock.Mock()
+        get_snapmirrors_mock = self.mock_object(
+            src_mock_client, 'get_snapmirror_destinations',
+            mock.Mock(side_effect=[['fake_snapmirror']]))
+        self.mock_object(src_mock_client, 'release_snapmirror_vol')
+
+        self.assertRaises(exception.NetAppException,
+                          self.dm_session.wait_for_snapmirror_release_vol,
+                          fake.VSERVER1, fake.VSERVER2, fake.SHARE_NAME,
+                          fake.SHARE_NAME2, False, src_mock_client,
+                          timeout=10)
+
+        get_snapmirrors_mock.assert_has_calls([
+            mock.call(source_vserver=fake.VSERVER1,
+                      dest_vserver=fake.VSERVER2,
+                      source_volume=fake.SHARE_NAME,
+                      dest_volume=fake.SHARE_NAME2)])
+        src_mock_client.release_snapmirror_vol.assert_called_once_with(
+            fake.VSERVER1, fake.SHARE_NAME, fake.VSERVER2, fake.SHARE_NAME2,
+            relationship_info_only=False)
+
+    @ddt.data([{'id': 'src_share'}, {'id': 'dst_share'}],
+              [{'id': 'dst_share'}])
+    def test_cleanup_previous_snapmirror_relationships(self, replica_list):
+        mock_src_client = mock.Mock()
+        src_backend_info = ('src_share', 'src_vserver', 'src_backend')
+        dst_backend_info = ('dst_share', 'dst_vserver', 'dst_backend')
+        self.mock_object(self.dm_session, 'get_backend_info_for_share',
+                         mock.Mock(side_effect=[src_backend_info,
+                                                dst_backend_info]))
+        self.mock_object(data_motion, 'get_client_for_backend',
+                         mock.Mock(return_value=mock_src_client))
+        self.mock_object(mock_src_client, 'release_snapmirror_vol')
+
+        result = self.dm_session.cleanup_previous_snapmirror_relationships(
+            {'id': 'src_share'}, replica_list)
+
+        data_motion.get_client_for_backend.assert_called_once_with(
+            'src_backend', vserver_name='src_vserver')
+        self.dm_session.get_backend_info_for_share.assert_has_calls([
+            mock.call({'id': 'src_share'}),
+            mock.call({'id': 'dst_share'})
+        ])
+        mock_src_client.release_snapmirror_vol.assert_called_once_with(
+            'src_vserver', 'src_share', 'dst_vserver', 'dst_share')
+
+        self.assertIsNone(result)
+
+    @ddt.data(netapp_api.NaApiError(),
+              netapp_api.NaApiError(code=netapp_api.EOBJECTNOTFOUND),
+              netapp_api.NaApiError(code=netapp_api.ESOURCE_IS_DIFFERENT),
+              netapp_api.NaApiError(code='some_random_code',
+                                    message="(entry doesn't exist)"),
+              netapp_api.NaApiError(code='some_random_code',
+                                    message='(actually, entry does exist!)'))
+    def test_cleanup_previous_snapmirror_relationships_does_not_exist(
+            self, release_exception):
+        mock_src_client = mock.Mock()
+        self.mock_object(self.dm_session, 'get_backend_info_for_share',
+                         mock.Mock(return_value=(
+                             mock.Mock(), mock.Mock(), mock.Mock())))
+        self.mock_object(data_motion, 'get_client_for_backend',
+                         mock.Mock(return_value=mock_src_client))
+        self.mock_object(mock_src_client, 'release_snapmirror_vol',
+                         mock.Mock(side_effect=release_exception))
+
+        replica = {'id': 'src_share'}
+        replica_list = [replica, {'id': 'dst_share'}]
+
+        result = self.dm_session.cleanup_previous_snapmirror_relationships(
+            replica, replica_list)
+
+        mock_src_client.release_snapmirror_vol.assert_called()
+        self.assertIsNone(result)
