@@ -82,13 +82,14 @@ class TestNexentaNasDriver(test.TestCase):
             return getattr(self.cfg, opt)
 
         self.cfg = mock.Mock(spec=conf.Configuration)
-        self.cfg.nexenta_host = '1.1.1.1'
+        self.cfg.nexenta_nas_host = '1.1.1.1'
         super(TestNexentaNasDriver, self).setUp()
 
         self.ctx = context.get_admin_context()
         self.cfg.safe_get = mock.Mock(side_effect=_safe_get)
         self.cfg.nexenta_rest_port = 1000
         self.cfg.reserved_share_percentage = 0
+        self.cfg.reserved_share_from_snapshot_percentage = 0
         self.cfg.max_over_subscription_ratio = 0
         self.cfg.nexenta_rest_protocol = 'auto'
         self.cfg.nexenta_volume = 'volume'
@@ -110,7 +111,7 @@ class TestNexentaNasDriver(test.TestCase):
         self.cfg.driver_handles_share_servers = False
 
         self.request_params = RequestParams(
-            'http', self.cfg.nexenta_host, self.cfg.nexenta_rest_port,
+            'http', self.cfg.nexenta_nas_host, self.cfg.nexenta_rest_port,
             '/rest/nms/', self.cfg.nexenta_user, self.cfg.nexenta_password)
 
         self.drv = nexenta_nas.NexentaNasDriver(configuration=self.cfg)
@@ -185,7 +186,8 @@ class TestNexentaNasDriver(test.TestCase):
         }
         self.cfg.nexenta_thin_provisioning = False
         path = '%s/%s/%s' % (self.volume, self.share, share['name'])
-        location = {'path': '%s:/volumes/%s' % (self.cfg.nexenta_host, path)}
+        location = {'path': '%s:/volumes/%s' % (
+            self.cfg.nexenta_nas_host, path)}
         post.return_value = FakeResponse()
 
         self.assertEqual([location],
@@ -268,7 +270,8 @@ class TestNexentaNasDriver(test.TestCase):
         snapshot = {'name': 'sn1', 'share_name': share['name']}
         post.return_value = FakeResponse()
         path = '%s/%s/%s' % (self.volume, self.share, share['name'])
-        location = {'path': '%s:/volumes/%s' % (self.cfg.nexenta_host, path)}
+        location = {'path': '%s:/volumes/%s' % (
+            self.cfg.nexenta_nas_host, path)}
         snapshot_name = '%s/%s/%s@%s' % (
             self.volume, self.share, snapshot['share_name'], snapshot['name'])
 
@@ -591,6 +594,8 @@ class TestNexentaNasDriver(test.TestCase):
                 'pool_name': 'volume',
                 'reserved_percentage': (
                     self.cfg.reserved_share_percentage),
+                'reserved_snapshot_percentage': (
+                    self.cfg.reserved_share_from_snapshot_percentage),
                 'compression': True,
                 'dedupe': True,
                 'thin_provisioning': self.cfg.nexenta_thin_provisioning,
